# CLOSE BLUE IRIS APPLICATION (NOT THE SERVICE) IF PC IS INAACTIVE (IDLE) FOR A PERIOD OF TIME
# AND OPTIONALLY SEND PUSHOVER NOTIFICATION
#
# Version: 1
#
# Created by jaydeel at ipcamtalk.com
#
# USAGE:      BI_app_stop_if_idle.ps1 [arg1]
# ARGUMENTS:  arg1 = [OPTIONAL} demo
#             'demo' mode is useful for simulating how the script works
#
# BEFORE USING THIS SCRIPT:
#   1. EDIT ALL VARIABLES IN 'USER SETTINGS' SECTION BELOW
#   2. COMMENT OUT UNUSED/UNNEEDED LINES
#
# HOW TO AUTOMATE THIS SCRIPT VIA TASK SCHEDULER
#   Open Task Scheduler and create an appropriately named task with one triggers and one action:
#   Task Scheduler
#   'General' tab
#        'Run only when user is logged in'  [REQUIRED] SEE NOTE #1
#        'Run with highest privileges'      [REQUIRED] SEE NOTE #2
#   'Triggers' tab:
#       'Daily' or 'One time' (e.g., at 12:05 AM, every 15 minutes)
#       (Note: I find 'One time' to be more reliable)
#   'Actions' tab:
#       Program/script: powershell.exe
#       Add arguments: {your_powershell_scripts_path)\{script_file_name}.ps1
#
# NOTES:
#   #1: This setting UNFORTUNATELY results in a flashing PowerShell window when the task runs.
#       While this issue can generally be avoided by running the task with setting:
#       General tab > 'Run whether user is logged in or not'
#	    this is not a viable option b/c it causes this script to fail to acquire the desired idle time. 
#       Therefore, we must use the specified setting.
#   #2: 'taskkill' requires administrative privileges to kill a task.
#
# REFS: 
#    https://stackoverflow.com/a/15846912/13499984 ... Win32 snippet used below
#    https://stackoverflow.com/a/50561117/13499984 ... tasklist output tios
#    https://www.ctrl.blog/entry/idle-task-scheduler-powershell.html
#    https://stackoverflow.com/a/1802836/13499984 ... hide Powershell window when run from Task Scheduler
#    https://superuser.com/a/1500462 ... taskkill /F option leaves icons in system tray
#

# MUST BE THE FIRST LINE IN A SCRIPT
param($p1)

# ================================================
# USER SETTINGS...

# ENTER IDLE TIME BEFORE CLOSE THE BLUE IRIS APP
# ... units are minutes
$idle_time_threshold_min = 30

# OPT-IN TO SEND PUSHOVER NOTIFICATION
# ... expecting $false or $true
$send_po_notification_flg = $false
#$send_po_notification_flg = $true

# ENTER PUSHOVER API USER KEY
# ... https://pushover.net/
$po_user = "YOUR_API_USER_KEY"

# ENTER PUSHOVER API APPLICATION TOKEN
# ... TIP: consider adding a 'PowerShell' application to your https://pushover.net/ account
$po_token = "YOUR_API_APPLICATION_TOKEN"

# ENTER PUSHOVER MESSAGE PRIORITY
# ... (integer) -2|-1|0|1|2; -2=lowest, -1=low(silent), 0=normal, 1=high-priority, 2=emergency.
# ... see https://pushover.net/api#priority
$po_priority = 0

# ENTER PUSHOVER SOUND
# ... blank ("") is default sound
# ... see https://pushover.net/api#sounds
$po_sound = ""  # default sound
#$po_sound = "siren"  # built-in sound
#$po_sound = "not-kiddin"  # custom sound

# ENTER PUSHOVER DEVICE(S)
# ... blank ("") sends to all devices
# ... may be comma-delimited list.
# ... see https://pushover.net/api
$po_device = ""  # all devices
#$po_device = "galaxy-s21"  # single device
#$po_device = "galaxy-s21,iPhoneXS"  # multiple devices

# ENTER PUSHOVER RETRY PARAMETER (SECONDS)
# ... Required for priority=2 messages (emergency)
# ... see https://pushover.net/api#priority
$po_retry = 30

# ENTER PUSHOVER EXPIRE PARAMETER (SECONDS)
# ... Required for priority=2 messages (emergency)
# ... see https://pushover.net/api#priority
$po_expire = 300

# ================================================

# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# Use Win32 API GetLastInputInfo
# ... Attribution - the source for this code section
#     Source: https://stackoverflow.com/a/15846912/13499984
Add-Type @'
using System;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace PInvoke.Win32 {

    public static class UserInput {

        [DllImport("user32.dll", SetLastError=false)]
        private static extern bool GetLastInputInfo(ref LASTINPUTINFO plii);

        [StructLayout(LayoutKind.Sequential)]
        private struct LASTINPUTINFO {
            public uint cbSize;
            public int dwTime;
        }

        public static DateTime LastInput {
            get {
                DateTime bootTime = DateTime.UtcNow.AddMilliseconds(-Environment.TickCount);
                DateTime lastInput = bootTime.AddMilliseconds(LastInputTicks);
                return lastInput;
            }
        }

        public static TimeSpan IdleTime {
            get {
                return DateTime.UtcNow.Subtract(LastInput);
            }
        }

        public static int LastInputTicks {
            get {
                LASTINPUTINFO lii = new LASTINPUTINFO();
                lii.cbSize = (uint)Marshal.SizeOf(typeof(LASTINPUTINFO));
                GetLastInputInfo(ref lii);
                return lii.dwTime;
            }
        }
    }
}
'@
# END - Use Win32 API GetLastInputInfo
# ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

function SendPushover($msg_str, $priority_level, $silent_flg) {
	msg.exe * /TIME:10 "priority_level = $priority_level`n$msg_str"
	$uri = 'https://api.pushover.net/1/messages.json'
	$parameters = @{
		token    = $script:po_token
		user     = $script:po_user
		priority = $priority_level
		retry    = $script:po_retry
		expire   = $script:po_expire
		sound    = $script:po_sound
		device   = $script:po_device
		message  = $msg_str
	}
	#$parameters | Invoke-RestMethod -Uri $uri -Method Post
	$parameters | Invoke-RestMethod -Uri $uri -Method Post | Out-Null
	if ($silent_flg -ne $true) {
		Write-Host "`nPushover notification sent...`n$msg"
	}
}

#$script_name = Split-Path -leaf $PSCommandpath
$script_name = $MyInvocation.MyCommand.Name

Write-Host "============================================================================="
Write-Host "Blue Iris Utility - Close Application (not service) if PC is Idle"
Write-Host "Note: use '$script_name demo' to simulate a 10 sec idle threshold"
Write-Host "-----------------------------------------------------------------------------"

# TEST FOR BLUE IRIS APPLICATION
# 'Tasklist' method... Ref: https://stackoverflow.com/a/50561117/13499984
# ... If the Blue Iris desktop application is NOT running, output is 
#     "INFO: No tasks are running which match the specified criteria."
$output = tasklist /FI "IMAGENAME eq blueiris.exe" /FI "SESSIONNAME ne Services" /FO CSV

# Retain for debugging.
<# tasklist /FI "IMAGENAME eq blueiris.exe" /FI "SESSIONNAME ne Services" /FO CSV
$msg = "BI PC tasklist... " + $output
#msg.exe * /TIME:10 $msg
#exit #>

# Retain for debugging
<# $dt = [PInvoke.Win32.UserInput]::LastInput
$idle_time_str = [PInvoke.Win32.UserInput]::IdleTime
$timespan = [TimeSpan]::Parse($idle_time_str)
#$idle_sec = $timespan.TotalSeconds
$idle_time_min = [math]::round($timespan.TotalMinutes,3)
$msg = "BI PC...`nlast input = " + $dt.ToLocalTime() + "`nidle time = $idle_time_str`nidle_time_min = $idle_time_min"
$prio = 0
SendPushover $msg $prio $false
exit #>

if ($output -eq 'INFO: No tasks are running which match the specified criteria.') {
	Write-Host ("Nothing to do - the Blue Iris desktop application is not running")
} else {
	if ($p1 -eq 'demo') {
		# DEMO MODE
		Write-Host ("Demo mode...")		
		$idle_time_threshold_sec = 10
		for ($i = 0; $i -lt 10; $i++) {
			# Get parameters
			$dt = [PInvoke.Win32.UserInput]::LastInput
			$idle_time_str = [PInvoke.Win32.UserInput]::IdleTime
			$timespan = [TimeSpan]::Parse($idle_time_str)
			$idle_sec = [math]::round($timespan.TotalSeconds,3)

			Write-Host ("Last input: " + $dt.ToLocalTime())
			Write-Host ("Idle for: " + $idle_time_str + " (" + $idle_sec + " sec)")
		
			if ($idle_sec -gt $idle_time_threshold_sec) {
				Write-Host ("idle time exceeds threshold (" + $idle_time_threshold_sec + " sec)")
				Write-Host ("*** Closing Blue Iris desktop application ***")

				# CLOSE BLUE IRIS APPLICATION
				# Choose one of the following... both work when executed from cmd.exe 
				#Get-Process BlueIris | Foreach-Object { $_.CloseMainWindow() | Out-Null } | stop-process -force
				# NOTE: We try a normal close before resorting to a hard close (/F); THE LATER LEAVES AN ICON IN SYSTEM TRAY.
				# REF: https://stackoverflow.com/a/58775751/13499984
				taskkill /IM BlueIris.exe /FI "SERVICES ne BlueIris"
				taskkill /IM BlueIris.exe /FI "SERVICES ne BlueIris" /F

				break
			}

			Start-Sleep -Seconds (Get-Random -Minimum 2 -Maximum 5)
		}
	} else {
		# NORMAL MODE
		
		# Get idle time of Blue Iris PC.
		$dt = [PInvoke.Win32.UserInput]::LastInput
		$idle_time_str = [PInvoke.Win32.UserInput]::IdleTime
		$timespan = [TimeSpan]::Parse($idle_time_str)
		$idle_time_min = [math]::round($timespan.TotalMinutes,3)

		Write-Host ("Last input: " + $dt.ToLocalTime())
		Write-Host ("Idle for: " + $idle_time_str + " (" + $idle_time_min + " min)")

		# Retain for debugging... HANDY WHEN TESTING FROM TASK SCHEDULER
		<# $msg = "BI PC...`nlast input = " + $dt.ToLocalTime() + "`nidle time = $idle_time_str`nidle_time_min = $idle_time_min"
		msg.exe * /TIME:10 $msg
		$prio = 0
		#SendPushover $msg $prio $false #>

		# Test for idle time trigger
		if ($idle_time_min -gt $idle_time_threshold_min) {

			# CLOSE BLUE IRIS APPLICATION
			# Choose one of the following... both work when executed from cmd.exe 
			#Get-Process BlueIris | Foreach-Object { $_.CloseMainWindow() | Out-Null } | stop-process -force
			# NOTE: We try a normal close before resorting to a hard close (/F); THE LATER LEAVES AN ICON IN SYSTEM TRAY.
			# REF: https://stackoverflow.com/a/58775751/13499984
			taskkill /IM BlueIris.exe /FI "SERVICES ne BlueIris"
			taskkill /IM BlueIris.exe /FI "SERVICES ne BlueIris" /F

			# PUSH NOTIFICATION (IF REQUESTED)
			if ($send_po_notification_flg) {
				$hhmm = Get-Date -Format HH:mm
				$msg = "HEADS UP: Blue Iris app closed @$hhmm`nidle time ($idle_time_min min) exceeds threshold ($idle_time_threshold_min)"
				SendPushover $msg $po_priority $false
				Write-Host "`nPushover notification sent...`n$msg"
			}
		} else {
			Write-Host ("Nothing to do - idle time below threshold ($idle_time_threshold_min min)")
		}
	}
}

Write-Host ""